/** \file
 * Header for the CAtlUUID class
 *
 * \author Abvent R&D Copyright (c) 2004
 * \author Richard Young
 * \version 1.0
 * \date 21/07/2004
 */


#ifndef CAtlUUID_h
#define CAtlUUID_h


#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif
#include <cstring>
#include <stdexcept>

// Define a cross platform UUID
#define AV_STD
#if defined(AVATL_CFM)
	#undef AV_STD
	#define AV_STD	::std
	#ifndef __CFUUID__
		#include <CFUUID.h>
	#endif
#elif defined(AVATL_MACHO)
	#ifndef __COREFOUNDATION_CFUUID__
		#include <CoreFoundation/CoreFoundation.h>
	#endif
#elif defined(AVATL_WIN)
	#include <objbase.h>
#else
	#error Please specify platform
#endif



namespace  AvAtl {


// Define a cross platform UUID
#if defined(AVATL_MAC)
	typedef CFUUIDBytes		XAtlUUID;
#elif defined(AVATL_WIN)
	typedef struct _GUID	XAtlUUID;
#else
	#error Please specify platform
#endif


/// Class that implement UUID
class CAtlUUID
{
public:
	typedef	char	uuidstr_t[33];

	/// Constructor
				CAtlUUID() { mUUID[0] = mUUID[1] = mUUID[2] = mUUID[3] = 0; }

	/// Constructor
				CAtlUUID(const XAtlUUID& inUUID) { AV_STD::memcpy(mUUID, &inUUID, sizeof(mUUID)); }

	/// Constructor
				CAtlUUID(const char* inUUIDStr)
				{
					unsigned char*	v = (unsigned char*)mUUID;
					for (unsigned int i = 0; i < 16; i++, inUUIDStr += 2)
						v[i] = (unsigned char)((Hex2Int(inUUIDStr[0]) << 4) | Hex2Int(inUUIDStr[1]));
				}

	/// Assignement operator
	CAtlUUID& 	operator =(const XAtlUUID& inUUID)
				{
					AV_STD::memcpy(mUUID, &inUUID, sizeof(mUUID));
					return *this;
				}

	/// Assignement
	void		assign(const CAtlUUID& inUUID)
				{
					AV_STD::memcpy(mUUID, inUUID.mUUID, sizeof (mUUID));
				}

	/// Auto convertion to plateform type
				operator const XAtlUUID&() const { return *(const XAtlUUID*)mUUID; }

	/// Set the outUUIDStr to the hex value of the UUID.
	void		toString(uuidstr_t outUUIDStr) const
				{
					static char	hexChars[17] = "0123456789ABCDEF";
					char*	s = outUUIDStr;
					const unsigned char*	v = (const unsigned char*)mUUID;
					for (unsigned int i = 0; i < 16; i++, s += 2)
					{
						s[0] = hexChars[v[i] >> 4];
						s[1] = hexChars[v[i] & 0x0f];
					}
					s[0] = 0;
				}

	// Return the hex value of the UUID
	std::string	toString() const
				{
					uuidstr_t	tmp;
					toString(tmp);
					return tmp;
				}

	/// Equality test
	bool		operator ==(const CAtlUUID& inUUID) const
				{
					return mUUID[0] == inUUID.mUUID[0] &&
						   mUUID[1] == inUUID.mUUID[1] &&
						   mUUID[2] == inUUID.mUUID[2] &&
						   mUUID[3] == inUUID.mUUID[3];
				}
	/// Difference test
	bool		operator !=(const CAtlUUID& inUUID) const { return !(*this == inUUID); }

	/// Return true if this object is greater that the other
	bool		operator >(const CAtlUUID& inUUID) const
				{
					return AV_STD::memcmp(mUUID, inUUID.mUUID, sizeof(mUUID)) > 0;
				}

	/// Return true if this object is little that the other
	bool		operator <(const CAtlUUID& inUUID) const
				{
					return AV_STD::memcmp(mUUID, inUUID.mUUID, sizeof(mUUID)) < 0;
				}

	/// Return true if the UUID isn't defined
	bool		IsNULL() const
				{
					return (mUUID[0] | mUUID[1] | mUUID[2] | mUUID[3]) == 0;
				}

	/// Generate a new UUID
	CAtlUUID&	NewUUID()
				{
#if defined(AVATL_MAC)
					CFUUIDRef	uuidRef = CFUUIDCreate(kCFAllocatorDefault);
					if (uuidRef == NULL)
						throw std::runtime_error("Can't create UUID");
					*(XAtlUUID*)mUUID = CFUUIDGetUUIDBytes(uuidRef);
					CFRelease(uuidRef);
#elif defined(AVATL_WIN)
					if (CoCreateGuid((XAtlUUID*)mUUID) != S_OK)
						throw std::runtime_error("Can't create UUID");
#else
	#error Please specify platform
#endif
					return *this;
				}

private:
	static int	Hex2Int(int inC)
				{
					int	v = 0;

					if (inC >= '0' && inC <= '9')
						v = inC - '0';
					else if (inC >= 'A' && inC <= 'F')
						v = 10 + inC - 'A';
					else if (inC >= 'a' && inC <= 'f')
						v = 10 + inC - 'a';
					else
						throw std::runtime_error("Invalid hex value");

					return v;
				}

	/// Plateform data
	unsigned int	mUUID[4];
};

} // namespace  AvAtl

#endif
