/**
\file IMappingNode.h

Interface class for mapping TCE node.

\verbatim

File            :	IMappingNode.h
Author			:	Richard Young
Creation date	:	28/04/2001
Description     :	This file is part of the API of the public ATL Library (CAtlMgrEx.h).
					The interface IMappingNode is the class for mapping node inside tce tree.
Modifications   :

\endverbatim
*/


#ifndef IMappingNode_h
#define IMappingNode_h


#ifndef CAtlVector_h
	#include "CATL/CAtlVector.h"
#endif
#ifndef CAtlColor_h
	#include "CATL/CAtlColor.h"
#endif
#ifndef AtlTypes_h
	#include "CATL/AvAtlTypes.h"
#endif


namespace AvAtl{


// Forward declarations.
class ITCENode;


/// The interface IMappingNode is the class for mapping node inside tce tree
/** \interface IMappingNode
 *
 * \author Abvent R&D Copyright (c) 2001-2012
 * \author Richard Young
 * \version 1.0
 * \date 28/04/2001
 */
class IMappingNode
{
public:
	// For ATL library internal usage only.
#ifdef ITCENode_h
	enum { mType = ITCENode::eMappingNode };
#endif

	enum EMappingGeometry
	{
		kMG_Planetary,			// Was PLANETARY_GEOMETRY
		kMG_Orthogonal,			// Was ORTHOGONAL_GEOMETRY
		kMG_Vertical,			// Was VERTICAL_GEOMETRY
		kMG_Spherical,			// Was SPHERICAL_GEOMETRY
		kMG_YCylindrical,		// Was YCYLINDRICAL_GEOMETRY
		kMG_XCylindrical,		// Was XCYLINDRICAL_GEOMETRY
		kMG_UVMapping,			// Was UV_GEOMETRY
		kMG_Horizontal
	};

	enum EMapType {
		kMT_Invalid,
		kMT_Diffuse,
		kMT_Specular,
		kMT_Glossy,	
		kMT_Reflection,
		kMT_Environment,
		kMT_Alpha,
		kMT_Bump,
		kMT_NormalMap,
		kMT_Neon,
		kMT_Movie,
		kMT_NbTypes
	};

	// Return the node of this mapping
	virtual const ITCENode*		asNode() const = 0;
	// Return the node of this mapping
	virtual ITCENode*			asNode() = 0;

	// Get the URL for the specified map type
	/** Empty string "" mean no map of this type */
	virtual tURL				getURL(EMapType inType) const = 0;
	// Set the URL for the specified map type
	/** Empty string "" mean no map of this type */
	virtual void				setURL(tURL inURL, EMapType inType) = 0;

	// Return true is this texture is from a shader definition
	virtual bool				getShaderFlag() const = 0;
	// Set to true is this texture is from a shader definition
	virtual void				setShaderFlag(bool inFlag) = 0;

	/// Return the number of maps linked to this one
	virtual unsigned int		getNbMapsLinked() const = 0;
	/// Return the UUID of the specified map linked
	/** return the node id of the linked map */
	virtual tUUID				getMapLinked(unsigned int inIndex) const = 0;

	/// Add a link to the specified map
	/** inUUID is the node id of the linked map */
	virtual void				addLinkedMap(tUUID inUUID) = 0;
	/// Remove a link to the specified map
	/** inUUID is the node id of the unlinked map */
	virtual void				removeLinkedMap(tUUID inUUID) = 0;

	/// Control the normal map value.
	virtual const CAtlVector&	getNormal() const = 0;
	virtual void				setNormal(const CAtlVector& inNormal) = 0;

	// Control the placement of the texture.
	virtual const CAtlVector&	getOrigin() const = 0;
	virtual void				setOrigin(const CAtlVector& inOrigin) = 0;

	// Control the placement of the texture.
	virtual const CAtlVector&	getXRefAxis() const = 0;
	virtual void				setXRefAxis(const CAtlVector& inXAxis) = 0;

	// Control the placement of the texture.
	virtual const CAtlVector&	getYRefAxis() const = 0;
	virtual void				setYRefAxis(const CAtlVector& inYAxis) = 0;

	/// Control the angle of the texture.
	virtual double				getAngle() const = 0;
	virtual void				setAngle(double inAngle) = 0;

	/// Control the size of the texture.
	virtual double				getHeight() const = 0;
	virtual void				setHeight(double inHeight) = 0;

	/// Control the size of the texture.
	virtual double				getWidth() const = 0;
	virtual void				setWidth(double inWidth) = 0;

	/// Control the bump level of the texture
	virtual double				getBump() const = 0;
	virtual void				setBump(double inBump) = 0;

	virtual double				getXCenter() const = 0;
	virtual void				setXCenter(double inCenterX) = 0;

	virtual double				getYCenter() const = 0;
	virtual void				setYCenter(double inCenterY) = 0;

	virtual double				getXDelta() const = 0;
	virtual void				setXDelta(double inXDelta) = 0;

	virtual double				getYDelta() const = 0;
	virtual void				setYDelta(double inYDelta) = 0;

	/// Control the specular power of the texture
	virtual double				getGlossy() const = 0;
	virtual void				setGlossy(double inGlossy) = 0;

	// Control the repetition of texture.
	virtual bool 				getXRepeatFlag() const = 0;
	virtual void				setXRepeatFlag(bool inRepeatX) = 0;

	// Control the repetition of texture.
	virtual bool 				getYRepeatFlag() const = 0;
	virtual void				setYRepeatFlag(bool inRepeatY) = 0;

	// Control the the specular color.
	virtual const CAtlColor&	getSpecularColor() const = 0;
	virtual void				setSpecularColor(const CAtlColor& inSpecColor) = 0;

	// Control the color to be the transparent color of texture.
	virtual const CAtlColor&	getTransparenceColor() const = 0;
	virtual void				setTransparenceColor(const CAtlColor& inColor) = 0;

	/// Control the reflection of the texture
	/** Each component can have a different reflection factor.
		Value for each component must be between 0 (no reflection) and 255 (full reflection).*/
	virtual const CAtlColor&	getReflectionColor() const = 0;
	virtual void				setReflectionColor(const CAtlColor& inColor) = 0;

	/// Control the reflection limit
	virtual double				getReflectionLimit() const = 0;
	virtual void				setReflectionLimit(double inReflectionLimit) = 0;

	virtual bool 				getCutSetFlag() const = 0;
	virtual void				setCutSetFlag(bool inFlag) = 0;

	virtual bool 				getCutoutFlag1() const = 0;
	virtual void				setCutoutFlag1(bool inFlag) = 0;

	virtual const CAtlColor&	getCutoutColor1() const = 0;
	virtual void				setCutoutColor1(const CAtlColor& inColor) = 0;

	virtual bool 				getCutoutFlag2() const = 0;
	virtual void				setCutoutFlag2(bool inFlag) = 0;

	virtual const CAtlColor&	getCutoutColor2() const = 0;
	virtual void				setCutoutColor2(const CAtlColor& inColor) = 0;

	// Control the invertion of texture.
	virtual bool 				getInvertXFlag() const = 0;
	virtual void				setInvertXFlag(bool inFlag) = 0;

	// Control the invertion of texture.
	virtual bool 				getInvertYFlag() const = 0;
	virtual void				setInvertYFlag(bool inFlag) = 0;

	virtual EMappingGeometry	getProjectionType() const = 0;
	virtual void				setProjectionType(EMappingGeometry inType) = 0;

	virtual bool				getPlanarFlag() const = 0;
	virtual void				setPlanarFlag(bool inFlag) = 0;

	virtual bool 				getVisibilityFlag() const = 0;
	virtual void				setVisibilityFlag(bool inFlag) = 0;

	virtual bool 				getMixFlag() const = 0;
	virtual void				setMixFlag(bool inFlag) = 0;

	virtual const CAtlColor&	getMixColor() const = 0;
	virtual void				setMixColor(const CAtlColor& inColor) = 0;

	virtual double				getMixValue() const = 0;
	virtual void				setMixValue(double inVal) = 0;

	virtual double				getAngularLODShift() const = 0;
	virtual void				setAngularLODShift(double inALS) = 0;

	virtual double				getLinearLODShift() const = 0;
	virtual void				setLinearLODShift(double inLLS) = 0;

	virtual bool 				getCutAlphaFlag() const = 0;
	virtual void				setCutAlphaFlag(bool inFlag) = 0;

	virtual bool 				getXMirrorFlag() const = 0;
	virtual void				setXMirrorFlag(bool inFlag) = 0;

	virtual bool 				getYMirrorFlag() const = 0;
	virtual void				setYMirrorFlag(bool inFlag) = 0;

	virtual bool 				getAlphaIsBumpFlag() const = 0;
	virtual void				setAlphaIsBumpFlag(bool inFlag) = 0;

	virtual bool 				getAlphaIsTransparencyFlag() const = 0;
	virtual void				setAlphaIsTransparencyFlag(bool inFlag) = 0;

	virtual bool 				getUseMipmapFlag() const = 0;
	virtual void				setUseMipmapFlag(bool inFlag) = 0;

	virtual bool 				getUseTrilinearFilterFlag() const = 0;
	virtual void				setUseTrilinearFilterFlag(bool inFlag) = 0;

	/// Methods for the display mode
	virtual void				setDisplayMode(bool inFlag) = 0;
	virtual bool				getDisplayMode() const = 0;

	virtual double				getBrightness() const = 0;
	virtual void				setBrightness(double inBrightness) = 0;

	virtual double				getContrast() const = 0;
	virtual void				setContrast(double inContrast) = 0;
	
	/// Methods for the movie mapping

	virtual int					getCurrentFrame() const = 0;
	virtual	void				setCurrentFrame(int inFrame) = 0;

	virtual int					getRealMovieLength() const = 0;
	virtual void				setRealMovieLength(int inLength) = 0;
	

	/// Get the normal mapping value
	virtual double				getNormalMapValue() const = 0;
	/// Set the normal mapping value
	virtual void				setNormalMapValue(double inValue) = 0;

	/// Get the id of this mapping node
	virtual tUUID				getId() const = 0;

	/// Set the id of this mapping node
	virtual void				setId(tUUID inId) = 0;
	
	/// Get the id of the parent mapping node
	virtual tUUID				getParentId() const = 0;
	
	/// Set the id of the parent mapping node
	virtual void				setParentId(tUUID inId) = 0;
	
	/** \retval the new XML node of the TCENode.
	\retval NULL if an error occurs. */
	virtual XMLNodeRef			putInXML(XMLNodeRef inFatherNode) const = 0;

	/// Initialize the TCENode with the XML.
	/** \retval true intialization from XML succeed.
		\retval false an error occurs. */
	virtual bool				initWithXML(ConstXMLNodeRef inFatherNode) = 0;

protected:
	// Don't delete, use node refcount
								~IMappingNode() {};
};


} // namespace  AvAtl

#endif
